var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { toCamelCase } from 'web-utility';
import { EChartsElement } from './renderers/core';
import { ProxyElement } from './Proxy';
import { callBus, unwrapEventHandler, wrapEventHandler } from './utility';
export class ECOptionElement extends ProxyElement {
    get chartTagName() {
        return toCamelCase(this.tagName.replace(/^ec-/i, '').toLowerCase());
    }
    get isSeries() {
        return this.chartTagName.endsWith('Chart');
    }
    get chartName() {
        return this.isSeries ? this.chartTagName.replace(/Chart$/, '') : null;
    }
    get eventSelector() {
        return [
            this.isSeries && 'series',
            this.chartName || this.chartTagName,
            this['type']
        ]
            .filter(Boolean)
            .join('.');
    }
    constructor() {
        super();
        Object.defineProperty(this, "renderer", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "updateOption", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: callBus(() => {
                const data = this.toJSON();
                const option = this.isSeries
                    ? { series: [Object.assign(Object.assign({}, data), { type: this.chartName })] }
                    : { [this.chartTagName]: data };
                this.renderer.setOption(option);
            })
        });
        Object.defineProperty(this, "addEventListener", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: callBus((name, handler) => {
                this.renderer.core.on(name, this.eventSelector, wrapEventHandler.call(this, name, handler));
            })
        });
        Object.defineProperty(this, "removeEventListener", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: callBus((event, handler) => {
                this.renderer.core.off(event, unwrapEventHandler(handler));
            })
        });
        this.updateOption.start();
        this.removeEventListener.start();
        this.addEventListener.start();
    }
    connectedCallback() {
        return __awaiter(this, void 0, void 0, function* () {
            for (let parent = this.parentElement; parent; parent = parent.parentElement)
                if (parent instanceof EChartsElement)
                    this.renderer = parent;
            if (!this.renderer)
                throw new ReferenceError(`<${this.tagName.toLowerCase()} /> should be append to a DOM tree within <ec-svg-renderer /> or <ec-canvas-renderer />`);
            yield this.renderer.ready;
            this.updateOption.run();
            this.removeEventListener.run();
            this.addEventListener.run();
        });
    }
    setProperty(key, value) {
        super.setProperty(key, value);
        this.updateOption();
    }
    setAttribute(name, value) {
        super.setAttribute(name, value);
        const key = toCamelCase(name);
        if (key in Object.getPrototypeOf(this))
            return;
        this[key] = name === value || value;
    }
}
//# sourceMappingURL=Option.js.map